@@ -9,7 +9,7 @@
  */
 
 #include "abi.h"
-#include "odb_hash.h"
+#include "odb.h"
 #include "op_sample_file.h"
 
 #include <iostream>
@@ -17,11 +17,11 @@
 
 using namespace std;
 
-typedef map<string,int> abi_map;
-typedef map<string,int>::const_iterator abi_iter;
+typedef map<string, int> abi_map;
+typedef map<string, int>::const_iterator abi_iter;
 
 #define byte_addr(x) (reinterpret_cast<unsigned char *>(&(x)))
-#define field_offset(s,f) (byte_addr(s.f) - byte_addr(s))
+#define field_offset(s, f) (byte_addr(s.f) - byte_addr(s))
 
 abi_exception::abi_exception(string const d) : desc(d) {}
 
@@ -62,6 +62,11 @@ abi::abi()
 	slots["offsetof_header_is_kernel"] = field_offset(header, is_kernel);
 	slots["offsetof_header_cpu_speed"] = field_offset(header, cpu_speed);
 	slots["offsetof_header_mtime"] = field_offset(header, mtime);
+	slots["offsetof_header_cg_to_is_kernel"] = field_offset(header,
+		cg_to_is_kernel);
+	slots["offsetof_header_anon_start"] = field_offset(header, anon_start);
+	slots["offsetof_header_cg_to_anon_start"] = field_offset(header,
+		cg_to_anon_start);
 
 	// determine endianness
 
