@@ -39,6 +39,18 @@ typedef enum {
 	CPU_P6_MOBILE, /**< Pentium M series */
 	CPU_ARM_XSCALE1, /**< ARM XScale 1 */
 	CPU_ARM_XSCALE2, /**< ARM XScale 2 */
+	CPU_PPC64_POWER4, /**< ppc64 POWER4 family */
+	CPU_PPC64_POWER5, /**< ppc64 POWER5 family */
+	CPU_PPC64_970, /**< ppc64 970 family */
+	CPU_MIPS_24K, /**< MIPS 24K */
+	CPU_MIPS_R10000, /**< MIPS R10000 */
+	CPU_MIPS_R12000, /**< MIPS R12000 */
+	CPU_MIPS_RM7000, /**< QED  RM7000 */
+	CPU_MIPS_RM9000, /**< PMC-Sierra RM9000 */
+	CPU_MIPS_SB1, /**< Broadcom SB1 */
+	CPU_MIPS_VR5432, /**< NEC VR5432 */
+	CPU_MIPS_VR5500, /**< MIPS VR5500, VR5532 and VR7701 */
+	CPU_PPC_E500,	/**< e500 */
 	MAX_CPU_TYPE
 } op_cpu;
 
