@@ -58,6 +58,17 @@ static int parse_hex(char const * str)
 }
 
 
+static u64 parse_long_hex(char const * str)
+{
+	u64 value;
+	if (sscanf(str, "%Lx", &value) != 1) {
+		parse_error("expected long hexadecimal value");
+	}
+	fflush(stderr);
+	return value;
+}
+
+
 /* name:MESI type:bitmask default:0x0f */
 static void parse_um(struct op_unit_mask * um, char const * line)
 {
@@ -452,7 +463,6 @@ static void load_events(op_cpu cpu_type)
 		check_unit_mask(um, cpu_name);
 	}
 	
-
 	free(um_file);
 	free(event_file);
 	free(event_dir);
@@ -509,7 +519,7 @@ void op_free_events(void)
 }
 
 
-struct op_event * find_event(u8 nr)
+static struct op_event * find_event(u8 nr)
 {
 	struct list_head * pos;
 
@@ -522,6 +532,140 @@ struct op_event * find_event(u8 nr)
 	return NULL;
 }
 
+
+static FILE * open_event_mapping_file(char const * cpu_name) 
+{
+	char * ev_map_file;
+	char * dir;
+	dir = getenv("OPROFILE_EVENTS_DIR");
+	if (dir == NULL)
+		dir = OP_DATADIR;
+
+	ev_map_file = xmalloc(strlen(dir) + strlen("/") + strlen(cpu_name) +
+	                    strlen("/") + + strlen("event_mappings") + 1);
+	strcpy(ev_map_file, dir);
+	strcat(ev_map_file, "/");
+
+	strcat(ev_map_file, cpu_name);
+	strcat(ev_map_file, "/");
+	strcat(ev_map_file, "event_mappings");
+	filename = ev_map_file;
+	return (fopen(ev_map_file, "r"));
+}
+
+
+/**
+ *  This function is PPC64-specific.
+ */
+static char const * get_mapping(u8 nr, FILE * fp) 
+{
+	char * line;
+	char * name;
+	char * value;
+	char const * c;
+	char * map = NULL;
+	int seen_event = 0, seen_mmcr0 = 0, seen_mmcr1 = 0, seen_mmcra = 0;
+	u32 mmcr0 = 0;
+	u64 mmcr1 = 0;
+	u32 mmcra = 0;
+	int event_found = 0;
+
+	line_nr = 1;
+	line = op_get_line(fp);
+	while (line && !event_found) {
+		if (empty_line(line) || comment_line(line))
+			goto next;
+
+		seen_event = 0;
+		seen_mmcr0 = 0;
+		seen_mmcr1 = 0;
+		seen_mmcra = 0;
+		mmcr0 = 0;
+		mmcr1 = 0;
+		mmcra = 0;
+
+		c = line;
+		while (next_token(&c, &name, &value)) {
+			if (strcmp(name, "event") == 0) {
+				u8 evt;
+				if (seen_event)
+					parse_error("duplicate event tag");
+				seen_event = 1;
+				evt = parse_hex(value);
+				if (evt == nr) {
+					event_found = 1;
+				}
+				free(value);
+			} else if (strcmp(name, "mmcr0") == 0) {
+				if (seen_mmcr0)
+					parse_error("duplicate mmcr0 tag");
+				seen_mmcr0 = 1;
+				mmcr0 = parse_hex(value);
+				free(value);
+			} else if (strcmp(name, "mmcr1") == 0) {
+				if (seen_mmcr1)
+					parse_error("duplicate mmcr1: tag");
+				seen_mmcr1 = 1;
+				mmcr1 = parse_long_hex(value);
+				free(value);
+			} else if (strcmp(name, "mmcra") == 0) {
+				if (seen_mmcra)
+					parse_error("duplicate mmcra: tag");
+				seen_mmcra = 1;
+				mmcra = parse_hex(value);
+				free(value);
+			} else {
+				parse_error("unknown tag");
+			}
+
+			free(name);
+		}
+next:
+		free(line);
+		line = op_get_line(fp);
+		++line_nr;
+	}
+	if (event_found) {
+		if (!seen_mmcr0 || !seen_mmcr1 || !seen_mmcra) {
+			fprintf(stderr, "Error: Missing information in line %d of event mapping file %s\n", line_nr, filename);
+			exit(EXIT_FAILURE);
+		}
+		map = xmalloc(70);
+		snprintf(map, 70, "mmcr0:%u mmcr1:%Lu mmcra:%u",
+		         mmcr0, mmcr1, mmcra);
+	}
+
+	return map;
+}
+
+
+char const * find_mapping_for_event(u8 nr, op_cpu cpu_type)
+{
+	char const * cpu_name = op_get_cpu_name(cpu_type);
+	FILE * fp = open_event_mapping_file(cpu_name);
+	char const * map = NULL;
+	switch (cpu_type) {
+		case CPU_PPC64_970:
+		case CPU_PPC64_POWER4:
+		case CPU_PPC64_POWER5:
+			if (!fp) {
+				fprintf(stderr, "oprofile: could not open event mapping file %s\n", filename);
+				exit(EXIT_FAILURE);
+			} else {
+				map = get_mapping(nr, fp);
+			}
+			break;			
+		default:
+			break;
+	}
+
+	if (fp)
+		fclose(fp);
+
+	return map;
+}
+
+
 struct op_event * find_event_by_name(char const * name)
 {
 	struct list_head * pos;
@@ -579,7 +723,7 @@ int op_check_events(int ctr, u8 nr, u16 um, op_cpu cpu_type)
 			if (event->unit->um[i].value == um)
 				break;
 		}
-		
+
 		if (i == event->unit->num)
 			ret |= OP_INVALID_UM;
 	}
@@ -588,18 +732,6 @@ int op_check_events(int ctr, u8 nr, u16 um, op_cpu cpu_type)
 }
 
 
-unsigned int op_min_count(u8 ctr_type, op_cpu cpu_type)
-{
-	struct op_event * event;
-
-	load_events(cpu_type);
-
-	event = find_event(ctr_type);
-	
-	return event ? event->min_count : 0;
-}
-
-
 void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 {
 	descr->name = "";
@@ -634,6 +766,7 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 		case CPU_IA64:
 		case CPU_IA64_1:
 		case CPU_IA64_2:
+			descr->count = 1000000;
 			descr->name = "CPU_CYCLES";
 			break;
 
@@ -651,6 +784,39 @@ void op_default_event(op_cpu cpu_type, struct op_default_event_descr * descr)
 			descr->name = "CPU_CYCLES";
 			break;
 
+		case CPU_PPC64_970:
+		case CPU_PPC64_POWER4:
+		case CPU_PPC64_POWER5:
+			descr->name = "CYCLES";
+			break;
+
+		case CPU_MIPS_24K:
+			descr->name = "INSTRUCTIONS";
+			break;
+
+		case CPU_MIPS_R10000:
+		case CPU_MIPS_R12000:
+			descr->name = "INSTRUCTIONS_GRADUATED";
+			break;
+
+		case CPU_MIPS_RM7000:
+		case CPU_MIPS_RM9000:
+			descr->name = "INSTRUCTIONS_ISSUED";
+			break;
+
+		case CPU_MIPS_SB1:
+			descr->name = "INSN_SURVIVED_STAGE7";
+			break;
+
+		case CPU_MIPS_VR5432:
+		case CPU_MIPS_VR5500:
+			descr->name = "INSTRUCTIONS_EXECUTED";
+			break;
+
+		case CPU_PPC_E500:
+			descr->name = "CPU_CLK";
+			break;
+             
 		// don't use default, if someone add a cpu he wants a compiler
 		// warning if he forgets to handle it here.
 		case CPU_TIMER_INT:
