@@ -79,18 +79,6 @@ bool operator<(profile_class const & lhs,
 
 namespace {
 
-/**
- * The "axis" says what we've used to split the sample
- * files into the classes. Only one is allowed.
- */
-enum axis_types {
-	AXIS_EVENT,
-	AXIS_TGID,
-	AXIS_TID,
-	AXIS_CPU,
-	AXIS_MAX
-};
-
 struct axis_t {
 	string name;
 	string suggestion;
@@ -102,12 +90,50 @@ struct axis_t {
 };
 
 
+bool profile_classes::matches(profile_classes const & classes)
+{
+	if (v.size() != classes.v.size())
+		return false;
+
+	axis_types const axis2 = classes.axis;
+
+	switch (axis) {
+		case AXIS_EVENT:
+			break;
+		case AXIS_TGID:
+		case AXIS_TID:
+			return axis2 == AXIS_TID || axis2 == AXIS_TGID;
+		case AXIS_CPU:
+			return axis2 == AXIS_CPU;
+		case AXIS_MAX:
+			return false;
+	}
+
+	// check that the events match (same event, count)
+
+	vector<profile_class>::const_iterator it1 = v.begin();
+	vector<profile_class>::const_iterator end1 = v.end();
+	vector<profile_class>::const_iterator it2 = classes.v.begin();
+
+	while (it1 != end1) {
+		if (it1->ptemplate.event != it2->ptemplate.event)
+			return false;
+		if (it1->ptemplate.count != it2->ptemplate.count)
+			return false;
+		// differing unit mask is considered comparable
+		++it1;
+		++it2;
+	}
+
+	return true;
+}
+
+
 /// We have more than one axis of classification, tell the user.
-void report_error(profile_classes const & classes,
-                  axis_types axis, axis_types newaxis)
+void report_error(profile_classes const & classes, axis_types newaxis)
 {
 	string str = "Already displaying results for parameter ";
-	str += axes[axis].name;
+	str += axes[classes.axis].name;
 	str += " with values:\n";
 	vector<profile_class>::const_iterator it = classes.v.begin();
 	vector<profile_class>::const_iterator const end = classes.v.end();
@@ -134,7 +160,7 @@ void report_error(profile_classes const & classes,
 	str += "\nwhich conflicts with parameter ";
 	str += axes[newaxis].name += ".\n";
 	str += "Suggestion: ";
-	str += axes[axis].suggestion;
+	str += axes[classes.axis].suggestion;
 	throw op_fatal_error(str);
 }
 
@@ -144,14 +170,13 @@ void report_error(profile_classes const & classes,
  * allowed if they are TGID,TID and for each class,
  * tid == tgid
  */
-bool allow_axes(profile_classes const & classes,
-                axis_types oldaxis, axis_types newaxis)
+bool allow_axes(profile_classes const & classes, axis_types newaxis)
 {
 	// No previous axis - OK
-	if (oldaxis == AXIS_MAX)
+	if (classes.axis == AXIS_MAX)
 		return true;
 
-	if (oldaxis != AXIS_TID && oldaxis != AXIS_TGID)
+	if (classes.axis != AXIS_TID && classes.axis != AXIS_TGID)
 		return false;
 
 	if (newaxis != AXIS_TID && newaxis != AXIS_TGID)
@@ -247,8 +272,7 @@ opd_header const get_header(profile_class const & pclass,
 
 
 /// Give human-readable names to each class.
-void name_classes(profile_classes & classes, axis_types axis,
-                  merge_option const & merge_by)
+void name_classes(profile_classes & classes, merge_option const & merge_by)
 {
 	opd_header header = get_header(classes.v[0], merge_by);
 
@@ -257,15 +281,15 @@ void name_classes(profile_classes & classes, axis_types axis,
 
 	// If we're splitting on event anyway, clear out the
 	// global event name
-	if (axis == 0)
+	if (classes.axis == AXIS_EVENT)
 		classes.event.erase();
 
 	vector<profile_class>::iterator it = classes.v.begin();
 	vector<profile_class>::iterator const end = classes.v.end();
 
 	for (; it != end; ++it) {
-		it->name = axes[axis].name + ":";
-		switch (axis) {
+		it->name = axes[classes.axis].name + ":";
+		switch (classes.axis) {
 		case AXIS_EVENT:
 			it->name = it->ptemplate.event
 				+ ":" + it->ptemplate.count;
@@ -332,20 +356,20 @@ void identify_classes(profile_classes & classes,
 			changed[AXIS_CPU] = true;
 	}
 
-	axis_types axis = AXIS_MAX;
+	classes.axis = AXIS_MAX;
 
 	for (size_t i = 0; i < AXIS_MAX; ++i) {
 		if (!changed[i])
 			continue;
 
-		if (!allow_axes(classes, axis, axis_types(i)))
-			report_error(classes, axis, axis_types(i));
-		axis = axis_types(i);
+		if (!allow_axes(classes, axis_types(i)))
+			report_error(classes, axis_types(i));
+		classes.axis = axis_types(i);
 		/* do this early for report_error */
-		name_classes(classes, axis, merge_by);
+		name_classes(classes, merge_by);
 	}
 
-	if (axis == AXIS_MAX) {
+	if (classes.axis == AXIS_MAX) {
 		cerr << "Internal error - no equivalence class axis" << endl;
 		abort();
 	}
@@ -393,6 +417,27 @@ profile_class & find_class(set<profile_class> & classes,
 	return const_cast<profile_class &>(*ret.first);
 }
 
+/**
+ * Sanity check : we can't overwrite sample_filename, if we abort here it means
+ * we fail to detect that parsed sample filename for two distinct samples
+ * filename must go in two distinct profile_sample_files. This assumption is
+ * false for callgraph samples files so this function is only called for non cg
+ * files.
+ */
+void sanitize_profile_sample_files(profile_sample_files const & sample_files,
+    parsed_filename const & parsed)
+{
+	// We can't allow to overwrite sample_filename.
+	if (!sample_files.sample_filename.empty()) {
+		ostringstream out;
+		out << "sanitize_profile_sample_files(): sample file "
+		    << "parsed twice ?\nsample_filename:\n"
+		    << sample_files.sample_filename << endl
+		    << parsed << endl;
+		throw op_fatal_error(out.str());
+	}
+}
+
 
 /**
  * Add a sample filename (either cg or non cg files) to this profile.
@@ -403,14 +448,8 @@ add_to_profile_sample_files(profile_sample_files & sample_files,
 {
 	if (parsed.cg_image.empty()) {
 		// We can't allow to overwrite sample_filename.
-		if (!sample_files.sample_filename.empty()) {
-			ostringstream out;
-			out << "add_to_profile_sample_files(): sample file "
-			    << "parsed twice ?\nsample_filename:\n"
-			    << sample_files.sample_filename << endl
-			    << parsed << endl;
-			throw op_fatal_error(out.str());
-		}
+		sanitize_profile_sample_files(sample_files, parsed);
+
 		sample_files.sample_filename = parsed.filename;
 	} else {
 		sample_files.cg_files.push_back(parsed.filename);
@@ -449,7 +488,7 @@ find_profile_sample_files(list<profile_sample_files> & files,
 		}
 	}
 
-	// not found create a new once.
+	// not found, create a new one
 	files.push_back(profile_sample_files());
 	return files.back();
 }
@@ -461,9 +500,9 @@ find_profile_sample_files(list<profile_sample_files> & files,
  * on the normal list of profiles otherwise.
  */
 void
-add_to_profile_set(profile_set & set, parsed_filename const & parsed)
+add_to_profile_set(profile_set & set, parsed_filename const & parsed, bool merge_by_lib)
 {
-	if (parsed.image == parsed.lib_image) {
+	if (parsed.image == parsed.lib_image && !merge_by_lib) {
 		profile_sample_files & sample_files =
 			find_profile_sample_files(set.files, parsed);
 		add_to_profile_sample_files(sample_files, parsed);
@@ -474,7 +513,7 @@ add_to_profile_set(profile_set & set, parsed_filename const & parsed)
 	list<profile_dep_set>::iterator const end = set.deps.end();
 
 	for (; it != end; ++it) {
-		if (it->lib_image == parsed.lib_image) {
+		if (it->lib_image == parsed.lib_image && !merge_by_lib) {
 			profile_sample_files & sample_files =
 				find_profile_sample_files(it->files, parsed);
 			add_to_profile_sample_files(sample_files, parsed);
@@ -496,21 +535,21 @@ add_to_profile_set(profile_set & set, parsed_filename const & parsed)
  * will have ensured the profile "fits", so now it's just a matter of
  * finding which sample file list it needs to go on.
  */
-void add_profile(profile_class & pclass, parsed_filename const & parsed)
+void add_profile(profile_class & pclass, parsed_filename const & parsed, bool merge_by_lib)
 {
 	list<profile_set>::iterator it = pclass.profiles.begin();
 	list<profile_set>::iterator const end = pclass.profiles.end();
 
 	for (; it != end; ++it) {
 		if (it->image == parsed.image) {
-			add_to_profile_set(*it, parsed);
+			add_to_profile_set(*it, parsed, merge_by_lib);
 			return;
 		}
 	}
 
 	profile_set set;
 	set.image = parsed.image;
-	add_to_profile_set(set, parsed);
+	add_to_profile_set(set, parsed, merge_by_lib);
 	pclass.profiles.push_back(set);
 }
 
@@ -540,7 +579,7 @@ arrange_profiles(list<string> const & files, merge_option const & merge_by)
 
 		profile_class & pclass =
 			find_class(temp_classes, parsed, merge_by);
-		add_profile(pclass, parsed);
+		add_profile(pclass, parsed, merge_by.lib);
 	}
 
 	profile_classes classes;
@@ -674,8 +713,8 @@ get_iprofile(app_map_t & app_map, string const & image, size_t nr_classes)
 
 /// Pull out all the images, removing any we can't access.
 void
-verify_and_fill(app_map_t & app_map, list<inverted_profile> & plist,
-                extra_images const & extra)
+verify_and_fill(string archive_path, app_map_t & app_map,
+		list<inverted_profile> & plist, extra_images const & extra)
 {
 	app_map_t::iterator it = app_map.begin();
 	app_map_t::iterator const end = app_map.end();
@@ -683,7 +722,8 @@ verify_and_fill(app_map_t & app_map, list<inverted_profile> & plist,
 	for (; it != end; ++it) {
 		plist.push_back(it->second);
 		inverted_profile & ip = plist.back();
-		ip.image = find_image_path(ip.image, extra, ip.error);
+		ip.image = find_image_path(archive_path, ip.image, extra,
+					   ip.error);
 	}
 }
 
@@ -691,7 +731,8 @@ verify_and_fill(app_map_t & app_map, list<inverted_profile> & plist,
 
 
 list<inverted_profile> const
-invert_profiles(profile_classes const & classes, extra_images const & extra)
+invert_profiles(string archive_path, profile_classes const & classes,
+		extra_images const & extra)
 {
 	app_map_t app_map;
 
@@ -729,7 +770,7 @@ invert_profiles(profile_classes const & classes, extra_images const & extra)
 
 	list<inverted_profile> inverted_list;
 
-	verify_and_fill(app_map, inverted_list, extra);
+	verify_and_fill(archive_path, app_map, inverted_list, extra);
 
 	return inverted_list;
 }
