@@ -16,7 +16,7 @@
 #include <sstream>
 
 #include "op_exception.h"
-#include "odb_hash.h"
+#include "odb.h"
 #include "op_cpu_type.h"
 #include "op_file.h"
 #include "op_header.h"
@@ -43,6 +43,13 @@ void op_check_header(opd_header const & h1, opd_header const & h2,
 		throw op_fatal_error(os.str());
 	}
 
+	if  (h1.anon_start != h2.anon_start) {
+		ostringstream os;
+		os << "header anon_start flags are different for "
+		   << filename << "\n";
+		throw op_fatal_error(os.str());
+	}
+	
 	// Note that we don't check CPU speed since that can vary
 	// freely on the same machine
 }
@@ -90,7 +97,7 @@ void check_mtime(string const & file, opd_header const & header)
 
 opd_header const read_header(string const & sample_filename)
 {
-	samples_odb_t samples_db;
+	odb_t samples_db;
 
 	int rc = odb_open(&samples_db, sample_filename.c_str(), ODB_RDONLY,
 		sizeof(struct opd_header));
@@ -98,7 +105,7 @@ opd_header const read_header(string const & sample_filename)
 	if (rc)
 		throw op_fatal_error(sample_filename + ": " + strerror(rc));
 
-	opd_header head = *static_cast<opd_header *>(samples_db.base_memory);
+	opd_header head = *static_cast<opd_header *>(samples_db.data->base_memory);
 
 	odb_close(&samples_db);
 
