@@ -22,8 +22,9 @@
 #include "op_header.h"
 #include "op_config.h"
 #include "op_sample_file.h"
-
 #include "profile.h"
+#include "op_bfd.h"
+#include "cverb.h"
 
 using namespace std;
 
@@ -36,7 +37,7 @@ profile_t::profile_t()
 // static member
 unsigned int profile_t::sample_count(string const & filename)
 {
-	samples_odb_t samples_db;
+	odb_t samples_db;
 
 	open_sample_file(filename, samples_db);
 
@@ -55,7 +56,7 @@ unsigned int profile_t::sample_count(string const & filename)
 }
 
 //static member
-void profile_t::open_sample_file(string const & filename, samples_odb_t & db)
+void profile_t::open_sample_file(string const & filename, odb_t & db)
 {
 	int rc = odb_open(&db, filename.c_str(), ODB_RDONLY,
 		sizeof(struct opd_header));
@@ -63,7 +64,8 @@ void profile_t::open_sample_file(string const & filename, samples_odb_t & db)
 	if (rc)
 		throw op_fatal_error(filename + ": " + strerror(rc));
 
-	opd_header const & head = *static_cast<opd_header *>(db.base_memory);
+	opd_header const & head =
+		*static_cast<opd_header *>(odb_get_data(&db));
 
 	if (head.version != OPD_VERSION) {
 		ostringstream os;
@@ -74,18 +76,18 @@ void profile_t::open_sample_file(string const & filename, samples_odb_t & db)
 	}
 }
 
-void profile_t::add_sample_file(string const & filename, u32 offset)
+void profile_t::add_sample_file(string const & filename)
 {
-	samples_odb_t samples_db;
+	odb_t samples_db;
 
 	open_sample_file(filename, samples_db);
 
-	opd_header const & head = *static_cast<opd_header *>(samples_db.base_memory);
+	opd_header const & head =
+		*static_cast<opd_header *>(odb_get_data(&samples_db));
 
 	// if we already read a sample file header pointer is non null
-	if (file_header.get()) {
+	if (file_header.get())
 		op_check_header(head, *file_header, filename);
-	}
 
 	file_header.reset(new opd_header(head));
 
@@ -107,23 +109,32 @@ void profile_t::add_sample_file(string const & filename, u32 offset)
 	}
 
 	odb_close(&samples_db);
+}
 
-	if (!get_header().is_kernel)
-		return;
 
-	start_offset = offset;
+void profile_t::set_offset(op_bfd const & abfd)
+{
+	opd_header const & header = get_header();
+	if (header.anon_start || header.is_kernel)
+		start_offset = abfd.get_start_offset(header.anon_start);
+	cverb << (vdebug) << "start_offset is now " << start_offset << endl;
 }
 
 
 profile_t::iterator_pair
 profile_t::samples_range(odb_key_t start, odb_key_t end) const
 {
-	// if the image contains no symbol the vma range is [0 - filesize]
-	// in this case we can't substract start_offset else we will underflow
-	// and the iterator range will be empty.
-	if (start)
-		start -= start_offset;
-
+	// Check the start position isn't before start_offset:
+	// this avoids wrapping/underflowing start/end.
+	// This can happen on e.g. ARM kernels, where .init is
+	// mapped before .text - we just have to skip any such
+	// .init symbols.
+	if (start < start_offset) {
+		return make_pair(const_iterator(ordered_samples.end(), 0), 
+			const_iterator(ordered_samples.end(), 0));
+	}
+	
+	start -= start_offset;
 	end -= start_offset;
 
 	// sanity check if start > end caller will enter into an infinite loop
