@@ -13,13 +13,19 @@
 #define SYMBOL_H
 
 #include "name_storage.h"
-#include "count_array.h"
+#include "growable_vector.h"
 #include "format_flags.h"
+#include "op_types.h"
 
 #include <bfd.h>
 
 #include <list>
 
+
+/// for storing sample counts
+typedef growable_vector<u32> count_array_t;
+
+
 /// A simple container for a fileno:linenr location.
 struct file_location {
 	file_location() : linenr(0) {}
@@ -74,4 +80,54 @@ struct symbol_entry {
 	column_flags output_hint(column_flags fl) const;
 };
 
+
+/// a collection of sorted symbols
+typedef std::vector<symbol_entry const *> symbol_collection;
+
+
+/**
+ * The public data for call-graph symbols. Each caller/callee has
+ * the sample counts replaced with the relevant arc counts, whilst
+ * the cg_symbol retains its self count.
+ */
+struct cg_symbol : public symbol_entry {
+	cg_symbol(symbol_entry const & sym) : symbol_entry(sym) {}
+
+	typedef std::vector<symbol_entry> children;
+
+	/// all callers of this symbol
+	children callers;
+	/// total count of callers
+	count_array_t total_caller_count;
+
+	/// all symbols called by this symbol
+	children callees;
+	/// total count of callees
+	count_array_t total_callee_count;
+};
+
+
+/// a collection of sorted callgraph symbols
+typedef std::vector<cg_symbol> cg_collection;
+
+
+/// for storing diff %ages
+typedef growable_vector<double> diff_array_t;
+
+
+/**
+ * Data for a diffed symbol.
+ */
+struct diff_symbol : public symbol_entry  {
+	diff_symbol(symbol_entry const & sym) : symbol_entry(sym) {}
+
+	/// diff %age values for each profile class
+	diff_array_t diffs;
+};
+
+
+/// a collection of diffed symbols
+typedef std::vector<diff_symbol> diff_collection;
+
+
 #endif /* !SYMBOL_H */
