@@ -12,13 +12,11 @@
 #ifndef SYMBOL_SORT_H
 #define SYMBOL_SORT_H
 
-#include "profile_container.h"
+#include "symbol.h"
 
 #include <vector>
 #include <string>
 
-class symbol_entry;
-
 struct sort_options {
 	enum sort_order {
 		// order give sort order if caller doesn't specify one
@@ -38,10 +36,22 @@ struct sort_options {
 	void add_sort_option(sort_order order);
 
 	/**
-	 * Sort the vector by the given criteria.
+	 * Sort the given container by the given criteria.
 	 */
 	void sort(symbol_collection & syms, bool reverse_sort,
-		  bool long_filenames) const;
+	          bool long_filenames) const;
+
+	/**
+	 * Sort the given container by the given criteria.
+	 */
+	void sort(cg_collection & syms, bool reverse_sort,
+	          bool long_filenames) const;
+
+	/**
+	 * Sort the given container by the given criteria.
+	 */
+	void sort(diff_collection & syms, bool reverse_sort,
+	          bool long_filenames) const;
 
 	std::vector<sort_order> options;
 };
