@@ -5,9 +5,9 @@
  * @remark Copyright 2002 OProfile authors
  * @remark Read the file COPYING
  *
- * @author John Levon <moz@compsoc.man.ac.uk>
- * @author Philippe Elie <phil_el@wanadoo.fr>
- * @author Will Cohen <wcohen@redhat.com>
+ * @author John Levon
+ * @author Philippe Elie
+ * @author Will Cohen
  */
 
 #include "oprofile.h"
@@ -20,10 +20,12 @@ __init op_cpu get_cpu_type(void)
 
 	/* FIXME: There should be a bit more checking here. */
 	switch (family) {
-	case 0x07: /* Itanium */
+	/* Itanium */
+	case 0x07:
 		return CPU_IA64_1;
 		break;
-	case 0x1f: /* Itanium 2 */
+	/* Itanium 2 */
+	case 0x1f:
 		return CPU_IA64_2;
 		break;
 	}
