@@ -15,7 +15,7 @@
 
 EXPORT_NO_SYMBOLS;
  
-MODULE_AUTHOR("John Levon (moz@compsoc.man.ac.uk)");
+MODULE_AUTHOR("John Levon (levon@movementarian.org)");
 MODULE_DESCRIPTION("Continuous Profiling Module");
 MODULE_LICENSE("GPL");
 
@@ -105,22 +105,25 @@ inline static void evict_op_entry(uint cpu, struct _oprof_data * data, long irq_
 	}
 }
 
-inline static void fill_op_entry(struct op_sample * ops, long eip, pid_t pid, int ctr)
+inline static void
+fill_op_entry(struct op_sample * ops, long eip, pid_t pid, pid_t tgid, int ctr)
 {
 	ops->eip = eip;
 	ops->pid = pid;
+	ops->tgid = tgid;
 	ops->counter = ctr;
 }
 
-void op_do_profile(uint cpu, long eip, long irq_enabled, int ctr)
+void fastcall op_do_profile(uint cpu, long eip, long irq_enabled, int ctr)
 {
 	struct _oprof_data * data = &oprof_data[cpu];
 	pid_t const pid = current->pid;
+	pid_t const tgid = op_get_tgid();
 	struct op_sample * samples = &data->buffer[data->nextbuf];
 
 	data->nr_irq++;
 
-	fill_op_entry(samples, eip, pid, ctr);
+	fill_op_entry(samples, eip, pid, tgid, ctr);
 	evict_op_entry(cpu, data, irq_enabled);
 }
 
@@ -173,6 +176,7 @@ inline static void up_and_check_note(void)
 			       "oprofile with a larger note buffer.\n");
 			warned = 1;
 		}
+		sysctl.nr_note_buffer_overflow++;
 		note_pos = sysctl.note_size - 1;
 	}
 
@@ -260,6 +264,7 @@ static int check_buffer_amount(int cpu_nr)
 		       "You must increase the module buffer size with\n"
 		       "opcontrol --setup --bufer-size= or reduce the "
 		       "interrupt frequency\n", num);
+		data->nr_buffer_overflow += num;
 		num = size;
 	} else
 		data->nextbuf = 0;
@@ -429,6 +434,7 @@ static int oprof_init_data(void)
 	ulong buf_size;
 	struct _oprof_data * data;
 
+	sysctl.nr_note_buffer_overflow = 0;
 	notebufsize = sizeof(struct op_note) * sysctl.note_size;
 	note_buffer = vmalloc(notebufsize);
  	if (!note_buffer) {
@@ -444,6 +450,7 @@ static int oprof_init_data(void)
 		data->buf_size = 0;
 		data->buffer = 0;
 		data->buf_watermark = 0;
+		data->nr_buffer_overflow = 0;
 	}
  
 	buf_size = (sizeof(struct op_sample) * sysctl.buf_size);
@@ -513,8 +520,7 @@ static int oprof_start(void)
 		goto out;
 	}
 
-	if (!sysctl.kernel_only)
-		op_intercept_syscalls();
+	op_intercept_syscalls();
 
 	int_ops->start();
 
@@ -527,15 +533,16 @@ out:
 
 /*
  * stop interrupts being generated and notes arriving.
- * This needs to be idempotent.
+ * This is idempotent.
  */
 static void oprof_partial_stop(void)
 {
 	BUG_ON(state == STOPPED);
 
-	op_restore_syscalls();
-
-	int_ops->stop();
+	if (state == RUNNING) {
+		op_restore_syscalls();
+		int_ops->stop();
+	}
 
 	state = STOPPING;
 }
@@ -597,7 +604,6 @@ static struct file_operations oprof_fops = {
  *                        notesize
  *                        dump
  *                        dump_stop
- *                        kernel_only
  *                        nr_interrupts
  *                        #ctr/
  *                          event
@@ -652,6 +658,27 @@ out:
 	return ret;
 }
 
+static int get_nr_buffer_overflow(ctl_table * table, int write, struct file * filp, void * buffer, size_t * lenp)
+{
+	uint cpu;
+	int ret = -EINVAL;
+
+	lock_sysctl();
+
+	if (write)
+		goto out;
+
+	for (cpu = 0 ; cpu < smp_num_cpus; cpu++) {
+		sysctl.nr_buffer_overflow += oprof_data[cpu].nr_buffer_overflow;
+		oprof_data[cpu].nr_buffer_overflow = 0;
+	}
+
+	ret =  proc_dointvec(table, write, filp, buffer, lenp);
+out:
+	unlock_sysctl();
+	return ret;
+}
+
 int lproc_dointvec(ctl_table * table, int write, struct file * filp, void * buffer, size_t * lenp)
 {
 	int err;
@@ -720,16 +747,17 @@ out:
 	return err;
 }
 
-static int const nr_oprof_static = 7;
+static int const nr_oprof_static = 8;
 
 static ctl_table oprof_table[] = {
 	{ 1, "bufsize", &sysctl_parms.buf_size, sizeof(int), 0644, NULL, &lproc_dointvec, NULL, },
 	{ 1, "dump", &sysctl_parms.dump, sizeof(int), 0666, NULL, &sysctl_do_dump, NULL, },
 	{ 1, "dump_stop", &sysctl_parms.dump_stop, sizeof(int), 0644, NULL, &sysctl_do_dump_stop, NULL, },
-	{ 1, "kernel_only", &sysctl_parms.kernel_only, sizeof(int), 0644, NULL, &lproc_dointvec, NULL, },
 	{ 1, "nr_interrupts", &sysctl.nr_interrupts, sizeof(int), 0444, NULL, &get_nr_interrupts, NULL, },
 	{ 1, "notesize", &sysctl_parms.note_size, sizeof(int), 0644, NULL, &lproc_dointvec, NULL, },
 	{ 1, "cpu_type", &sysctl.cpu_type, sizeof(int), 0444, NULL, &lproc_dointvec, NULL, },
+	{ 1, "note_buffer_overflow", &sysctl.nr_note_buffer_overflow, sizeof(int), 0444, NULL, &lproc_dointvec, NULL, },
+	{ 1, "buffer_overflow", &sysctl.nr_buffer_overflow, sizeof(int), 0444, NULL, &get_nr_buffer_overflow, NULL, },
 	{ 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, },
 	{ 0, },
 };
@@ -811,7 +839,7 @@ int __init oprof_init(void)
 	if ((err = init_sysctl()))
 		goto out_err;
 
- 	err = op_major = register_chrdev(0, "oprof", &oprof_fops);
+	err = op_major = register_chrdev(0, "oprof", &oprof_fops);
 	if (err < 0)
 		goto out_err2;
 
