@@ -16,8 +16,10 @@
 #include <fstream>
 #include <utility>
 
+#include "op_exception.h"
 #include "op_header.h"
 #include "profile.h"
+#include "populate.h"
 #include "op_sample_file.h"
 #include "cverb.h"
 #include "string_manip.h"
@@ -25,20 +27,20 @@
 #include "child_reader.h"
 #include "op_file.h"
 #include "file_manip.h"
-#include "partition_files.h"
+#include "arrange_profiles.h"
 #include "opannotate_options.h"
 #include "profile_container.h"
 #include "symbol_sort.h"
+#include "image_errors.h"
 
 using namespace std;
 using namespace options;
 
 namespace {
 
-scoped_ptr<profile_container> samples;
+size_t nr_events;
 
-// needed to display samples file information
-scoped_ptr<opd_header> header;
+scoped_ptr<profile_container> samples;
 
 /// how opannotate was invoked
 string cmdline;
@@ -54,58 +56,20 @@ string const end_comment(" */");
 /// field width for the sample count
 unsigned int const count_width = 6;
 
-
-// FIXME share with opgprof.cpp and opreport.cpp
-image_set populate_samples(profile_container & samples,
-			   partition_files const & files,
-			   bool merge_lib)
-{
-	image_set images = sort_by_image(files, extra_found_images);
-
-	image_set::const_iterator it;
-	for (it = images.begin(); it != images.end(); ) {
-		pair<image_set::const_iterator, image_set::const_iterator>
-			p_it = images.equal_range(it->first);
-
-		op_bfd abfd(p_it.first->first, symbol_filter);
-		profile_t profile;
-
-		string app_name = p_it.first->second.image;
-		if (merge_lib) {
-			app_name = p_it.first->first;
-		}
-
-		for (; it != p_it.second; ++it) {
-			profile.add_sample_file(it->second.sample_filename,
-						abfd.get_start_offset());
-		}
-
-		check_mtime(abfd.get_filename(), profile.get_header());
-	
-		samples.add(profile, abfd, app_name);
-	}
-
-	return images;
-}
-
-
-void save_sample_file_header(partition_files const & files)
-{
-	if (files.nr_set()) {
-		partition_files::filename_set const & file_set = files.set(0);
-		opd_header temp = read_header(file_set.begin()->sample_filename);
-		header.reset(new opd_header(temp));
-	}
-}
-
-
 string get_annotation_fill()
 {
 	string str;
 
-	str += string(count_width, ' ') + ' ';
-	str += string(percent_width, ' '); 
+	for (size_t i = 0; i < nr_events; ++i) {
+		str += string(count_width, ' ') + ' ';
+		str += string(percent_width, ' ');
+	}
 
+	for (size_t i = 1; i < nr_events; ++i) {
+		str += "  ";
+	}
+
+	str += " :";
 	return str;
 }
 
@@ -137,23 +101,11 @@ void output_info(ostream & out)
 		out << in_comment
 		    << "Output annotated source file with samples" << '\n';
 
-		// FIXME: re-add this
-#if 0
-		if (threshold_percent != 0) {
-			if (!do_until_more_than_samples) {
-				out << in_comment
-				    << "Output files where the selected counter reach "
-				    << threshold_percent << "% of the samples"
-				    << '\n';
-			} else {
-				out << in_comment << "output files until "
-				    << threshold_percent
-				    << "% of the samples is reached on the selected counter"
-				    << '\n';
-			}
-		} else
-#endif
-		{
+		if (options::threshold != 0) {
+			out << in_comment
+			    << "Output files where samples count reach "
+			    << options::threshold << "% of the samples\n";
+		} else {
 			out << in_comment << "Output all files" << '\n';
 		}
 	} else {
@@ -172,27 +124,27 @@ void output_info(ostream & out)
 
 	out << in_comment << '\n';
 
-	stringstream stream;
+	out << in_comment << classes.cpuinfo << endl;
+	if (!classes.event.empty())
+		out << in_comment << classes.event << endl;
 
-	stream << *header;
-	stream.seekp(0);
-
-	string line;
-	while (getline(stream, line)) {
-		out << in_comment << line << '\n';
-	}
+	for (size_t i = 0; i < classes.v.size(); ++i)
+		out << in_comment << classes.v[i].longname << endl;
 
 	out << end_comment << '\n';
 }
 
 
-string counter_str(size_t counter, size_t total)
+string count_str(count_array_t const & count,
+		   count_array_t const & total)
 {
 	ostringstream os;
-	os << setw(count_width) << counter << ' ';
+	for (size_t i = 0; i < nr_events; ++i) {
+		os << setw(count_width) << count[i] << ' ';
 
-	os << format_double(op_ratio(counter, total) * 100.0,
-			    percent_int_width, percent_fract_width);
+		os << format_percent(op_ratio(count[i], total[i]) * 100.0,
+				    percent_int_width, percent_fract_width);
+	}
 	return os.str();
 }
 
@@ -205,18 +157,22 @@ string asm_line_annotation(symbol_entry const * last_symbol,
 	//  - we does not need cross architecture compile so the native
 	// strtoull must work, assuming unsigned long long can contain a vma
 	// and on 32/64 bits box bfd_vma is 64 bits
-	bfd_vma vma = strtoull(value.c_str(), NULL, 16);
+	// gcc 2.91.66 workaround
+	bfd_vma vma = 0;
+	vma = strtoull(value.c_str(), NULL, 16);
 
 	string str;
 
 	sample_entry const * sample = samples->find_sample(last_symbol, vma);
-	if (sample)
-		str += counter_str(sample->count, samples->samples_count());
-
-	if (str.empty())
+	if (sample) {
+		str += count_str(sample->counts, samples->samples_count());
+		for (size_t i = 1; i < nr_events; ++i)
+			str += "  ";
+		str += " :";
+	} else {
 		str = annotation_fill;
+	}
 
-	str += " :";
 	return str;
 }
 
@@ -226,16 +182,16 @@ string symbol_annotation(symbol_entry const * symbol)
 	if (!symbol)
 		return string();
 
-	string annot = counter_str(symbol->sample.count,
-				   samples->samples_count());
+	string annot = count_str(symbol->sample.counts,
+	                         samples->samples_count());
 	if (annot.empty())
-		return  string();
+		return string();
 
 	string const & symname = symbol_names.demangle(symbol->name);
 
 	string str = " ";
 	str += begin_comment + symname + " total: ";
-	str += counter_str(symbol->sample.count, samples->samples_count());
+	str += count_str(symbol->sample.counts, samples->samples_count());
 	str += end_comment;
 	return str;
 }
@@ -273,7 +229,7 @@ symbol_entry const * output_objdump_asm_line(symbol_entry const * last_symbol,
 
 	if (pos == str.length() || !isxdigit(str[pos])) {
 		if (do_output) {
-			cout << annotation_fill << " :" << str << '\n';
+			cout << annotation_fill << str << '\n';
 			return last_symbol;
 		}
 	}
@@ -283,7 +239,7 @@ symbol_entry const * output_objdump_asm_line(symbol_entry const * last_symbol,
 
 	if (pos == str.length() || (!isspace(str[pos]) && str[pos] != ':')) {
 		if (do_output) {
-			cout << annotation_fill << " :" << str << '\n';
+			cout << annotation_fill << str << '\n';
 			return last_symbol;
 		}
 	}
@@ -369,7 +325,8 @@ void do_one_output_objdump(symbol_collection const & symbols,
 		return ;
 	}
 
-	reader.terminate_process();  // force error code to be acquired
+	// force error code to be acquired
+	reader.terminate_process();
 
 	// required because if objdump stop by signal all above things suceeed
 	// (signal error message are not output through stdout/stderr)
@@ -433,14 +390,16 @@ string const source_line_annotation(debug_name_id filename, size_t linenr)
 {
 	string str;
 
-	u32 count = samples->samples_count(filename, linenr);
-	if (count)
-		str += counter_str(count, samples->samples_count());
-
-	if (str.empty())
+	count_array_t counts = samples->samples_count(filename, linenr);
+	if (!counts.zero()) {
+		str += count_str(counts, samples->samples_count());
+		for (size_t i = 1; i < nr_events; ++i)
+			str += "  ";
+		str += " :";
+	} else {
 		str = annotation_fill;
+	}
 
-	str += " :";
 	return str;
 }
 
@@ -454,14 +413,14 @@ string source_symbol_annotation(debug_name_id filename, size_t linenr)
 
 
 void output_per_file_info(ostream & out, debug_name_id filename,
-			  u32 total_count_for_file)
+			  count_array_t const & total_file_count)
 {
 	out << begin_comment << '\n'
 	     << in_comment << "Total samples for file : "
 	     << '"' << debug_names.name(filename) << '"'
 	     << '\n';
 	out << in_comment << '\n' << in_comment
-	    << counter_str(total_count_for_file, samples->samples_count())
+	    << count_str(total_file_count, samples->samples_count())
 	    << '\n';
 	out << end_comment << '\n' << '\n';
 }
@@ -480,9 +439,9 @@ string const line0_info(debug_name_id filename)
 
 
 void do_output_one_file(ostream & out, istream & in, debug_name_id filename,
-			bool header)
+                        bool header)
 {
-	u32 count = samples->samples_count(filename);
+	count_array_t count = samples->samples_count(filename);
 
 	if (header) {
 		output_per_file_info(out, filename, count);
@@ -518,48 +477,56 @@ void do_output_one_file(ostream & out, istream & in, debug_name_id filename,
 
 
 void output_one_file(istream & in, debug_name_id filename,
-		     bool output_separate_file)
+                     string const & source)
 {
-	if (!output_separate_file) {
+	if (output_dir.empty()) {
 		do_output_one_file(cout, in, filename, true);
 		return;
 	}
 
-	string out_filename = debug_names.name(filename);
-	string original_out_filename = out_filename;
+	string const out_file = op_realpath(output_dir + source);
 
-	size_t pos = out_filename.find(source_dir);
-	if (pos == 0) {
-		out_filename.erase(0, source_dir.length());
-	} else if (pos == string::npos) {
-		// filename is outside the source dir: ignore this file
-		cerr << "opannotate: file "
-		     << '"' << out_filename << '"' << " ignored" << endl;
+	/* Just because you're paranoid doesn't mean they're not out to
+	 * get you ...
+	 *
+	 * This is just a lame final safety check. If we found the
+	 * source, then "source" should be canonical already, and
+	 * can't escape from the output dir. We can't use op_realpath()
+	 * alone as that needs the file to exist already.
+	 *
+	 * Let's not complain again if we couldn't find the file anyway.
+	 */
+	if (out_file.find("/../") != string::npos) {
+		if (in) {
+			cerr << "refusing to create non-canonical filename "
+			     << out_file  << endl;
+		}
+		return;
+	} else if (!is_prefix(out_file, output_dir)) {
+		if (in) {
+			cerr << "refusing to create file " << out_file
+			     << " outside of output directory " << output_dir
+			     << endl;
+		}
 		return;
 	}
 
-	out_filename = relative_to_absolute_path(output_dir + out_filename);
-
-	if (create_path(out_filename.c_str())) {
-		cerr << "unable to create directory: "
-		     << '"' << dirname(out_filename) << '"' << endl;
+	if (is_files_identical(out_file, source)) {
+		cerr << "input and output files are identical: "
+		     << out_file << endl;
 		return;
 	}
 
-	// paranoid checking: out_filename and filename must be distinct file.
-	if (is_files_identical(original_out_filename, out_filename)) {
-		cerr << "input and output_filename are identical: "
-		     << '"' << original_out_filename << '"'
-		     << ','
-		     << '"' << out_filename << '"'
-		     << endl;
+	if (create_path(out_file.c_str())) {
+		cerr << "unable to create file: "
+		     << '"' << op_dirname(out_file) << '"' << endl;
 		return;
 	}
 
-	ofstream out(out_filename.c_str());
+	ofstream out(out_file.c_str());
 	if (!out) {
 		cerr << "unable to open output file "
-		     << '"' << out_filename << '"' << endl;
+		     << '"' << out_file << '"' << endl;
 	} else {
 		do_output_one_file(out, in, filename, false);
 		output_info(out);
@@ -567,84 +534,118 @@ void output_one_file(istream & in, debug_name_id filename,
 }
 
 
-void output_source(path_filter const & filter, bool output_separate_file)
+/* Locate a source file from debug info, which may be relative */
+string const locate_source_file(debug_name_id filename_id)
 {
-	if (!output_separate_file)
+	string const origfile = debug_names.name(filename_id);
+	string file = origfile;
+
+	if (file.empty())
+		return file;
+
+	/* Allow absolute paths to be relocated to a different directory */
+	if (file[0] == '/') {
+		vector<string>::const_iterator cit = base_dirs.begin();
+		vector<string>::const_iterator end = base_dirs.end();
+		for (; cit != end; ++cit) {
+			string path = op_realpath(*cit);
+
+			if (is_prefix(file, path)) {
+				file = file.substr(path.length());
+				break;
+			}
+		}
+	}
+
+	vector<string>::const_iterator cit = search_dirs.begin();
+	vector<string>::const_iterator end = search_dirs.end();
+
+	for (; cit != end; ++cit) {
+		string const absfile = op_realpath(*cit + "/" + file);
+
+		if (op_file_readable(absfile)) {
+			return absfile;
+		}
+	}
+
+	/* We didn't find a relocated absolute file, or a relative file,
+	 * assume the original is correct, accounting for the
+	 * possibility it's relative the cwd
+	 */
+	return op_realpath(origfile);
+}
+
+
+void output_source(path_filter const & filter)
+{
+	bool const separate_file = !output_dir.empty();
+
+	if (!separate_file)
 		output_info(cout);
 
 	vector<debug_name_id> filenames =
 		samples->select_filename(options::threshold);
 
 	for (size_t i = 0 ; i < filenames.size() ; ++i) {
-		string filename = debug_names.name(filenames[i]);
-		if (!filter.match(filename))
+		string const & source = locate_source_file(filenames[i]);
+
+		if (!filter.match(source))
 			continue;
 
-		ifstream in(filename.c_str());
+		ifstream in(source.c_str());
 
-		if (!in) {
-			// it is common to have empty filename due to the lack
-			// of debug info (eg _init function) so warn only
-			// if the filename is non empty. The case: no debug
-			// info at all has already been checked.
-			if (filename.length())
-				cerr << "opannotate (warning): unable to "
-				     << "open for reading: "
-				     << filename << endl;
-		} 
+		// it is common to have empty filename due to the lack
+		// of debug info (eg _init function) so warn only
+		// if the filename is non empty. The case: no debug
+		// info at all has already been checked.
+		if ((!in) && source.length()) {
+			cerr << "opannotate (warning): unable to open for "
+			     "reading: " << source << endl;
+		}
 
-		if (filename.length()) {
-			output_one_file(in, filenames[i], output_separate_file);
+		if (source.length()) {
+			output_one_file(in, filenames[i], source);
 		}
 	}
 }
 
 
-bool annotate_source(image_set const & images)
+bool annotate_source(list<string> const & images)
 {
 	annotation_fill = get_annotation_fill();
 
-	bool output_separate_file = false;
-	if (!source_dir.empty()) {
-		output_separate_file = true;
-
-		source_dir = relative_to_absolute_path(source_dir);
-		if (source_dir.length() &&
-		    source_dir[source_dir.length() - 1] != '/')
-			source_dir += '/';
-	}
-
-	if (!output_dir.empty() || output_separate_file) {
-		output_separate_file = true;
-
-		output_dir = relative_to_absolute_path(output_dir);
-		if (output_dir.length() &&
-		    output_dir[output_dir.length() - 1] != '/')
-			output_dir += '/';
-
+	if (!output_dir.empty()) {
 
 		if (create_path(output_dir.c_str())) {
 			cerr << "unable to create " << output_dir
 			     << " directory: " << endl;
 			return false;
 		}
-	}
 
-	if (output_separate_file && output_dir == source_dir) {
-		cerr << "You cannot specify the same directory for "
-		     << "--output-dir and --source-dir" << endl;
-		return false;
+		// Make sure we have an absolute path.
+		output_dir = op_realpath(output_dir);
+		if (output_dir.length() &&
+		    output_dir[output_dir.length() - 1] != '/')
+			output_dir += '/';
+
+		/* Don't let the user stomp on their sources */
+		if (output_dir == "/") {
+			cerr << "Output path of / would over-write the "
+				"source files" << endl;
+			return false;
+		}
 	}
 
 	if (assembly) {
 		bool some_output = false;
-		image_set::const_iterator it;
-		for (it = images.begin(); it != images.end(); ) {
-			if (output_asm(it->first)) {
+
+		list<string>::const_iterator it = images.begin();
+		list<string>::const_iterator const end = images.end();
+
+		for (; it != end; ++it) {
+			if (output_asm(*it)) {
 				some_output = true;
 			}
-
-			it = images.upper_bound(it->first);
 		}
 
 		if (!some_output) {
@@ -654,23 +655,48 @@ bool annotate_source(image_set const & images)
 			     << "the selected symbol\n";
 		}
 	} else {
-		output_source(file_filter, output_separate_file);
+		output_source(file_filter);
 	}
 
 	return true;
 }
 
 
-int opannotate(vector<string> const & non_options)
+int opannotate(options::spec const & spec)
 {
-	handle_options(non_options);
+	handle_options(spec);
 
-	samples.reset(new profile_container(false, true, true));
+	nr_events = classes.v.size();
 
-	save_sample_file_header(*sample_file_partition);
+	samples.reset(new profile_container(true, true));
+
+	list<string> images;
+
+	list<inverted_profile> iprofiles
+		= invert_profiles(options::archive_path, classes,
+				  options::extra_found_images);
+
+	report_image_errors(iprofiles);
+
+	list<inverted_profile>::iterator it = iprofiles.begin();
+	list<inverted_profile>::iterator const end = iprofiles.end();
+
+	bool debug_info = false;
+	for (; it != end; ++it) {
+		bool tmp = false;
+		populate_for_image(options::archive_path, *samples, *it,
+		                   options::symbol_filter, &tmp);
+		images.push_back(it->image);
+		if (tmp)
+			debug_info = true;
+	}
+
+	if (!debug_info && !options::assembly) {
+		cerr << "no debug information available for any binary "
+		     << "selected and --assembly not requested\n";
+		exit(EXIT_FAILURE);
+	}
 
-	image_set images = populate_samples(*samples, *sample_file_partition,
-					    false);
 	annotate_source(images);
 
 	return 0;
