@@ -14,27 +14,24 @@
 
 #include <string>
 
-#include "utility.h"
 #include "common_option.h"
 
-class partition_files;
-
 namespace options {
+	extern std::string archive_path;
 	extern std::string gmon_filename;
 }
 
-/**
- * a partition of sample filename to treat, each sub-list is a list of
- * sample to merge. filled by handle_options()
- */
-extern scoped_ptr<partition_files> sample_file_partition;
+class inverted_profile;
+
+/// a set of sample filenames to handle.
+extern inverted_profile image_profile;
 
 /**
  * handle_options - process command line
- * @param non_options vector of non options string
+ * @param spec  profile specification
  *
- * Process the arguments, fatally complaining on error.
+ * Process the spec, fatally complaining on error.
  */
-void handle_options(std::vector<std::string> const & non_options);
+void handle_options(options::spec const & spec);
 
 #endif // OPGPROF_OPTIONS_H
