@@ -26,6 +26,8 @@
 #include "populate.h"
 #include "arrange_profiles.h"
 #include "profile_container.h"
+#include "callgraph_container.h"
+#include "diff_container.h"
 #include "symbol_sort.h"
 #include "format_output.h"
 #include "image_errors.h"
@@ -358,6 +360,9 @@ format_flags const get_format_flags(column_flags const & cf)
 		flags = format_flags(flags | ff_percent_cumulated);
 	}
 
+	if (classes2.v.size())
+		flags = format_flags(flags | ff_diff);
+
 	if (cf & cf_image_name)
 		flags = format_flags(flags | ff_image_name);
 
@@ -365,18 +370,17 @@ format_flags const get_format_flags(column_flags const & cf)
 }
 
 
-void output_symbols(profile_container const & samples, bool multiple_apps)
+void output_symbols(profile_container const & pc, bool multiple_apps)
 {
 	profile_container::symbol_choice choice;
 	choice.threshold = options::threshold;
-	symbol_collection symbols = samples.select_symbols(choice);
+	symbol_collection symbols = pc.select_symbols(choice);
 	options::sort_by.sort(symbols, options::reverse_sort,
 	                      options::long_filenames);
 
-	format_output::opreport_formatter out(samples);
+	format_output::opreport_formatter out(pc);
 
 	out.set_nr_classes(nr_classes);
-
 	out.show_details(options::details);
 	out.show_long_filenames(options::long_filenames);
 	out.show_header(options::show_header);
@@ -392,9 +396,65 @@ void output_symbols(profile_container const & samples, bool multiple_apps)
 }
 
 
-int opreport(vector<string> const & non_options)
+void output_diff_symbols(profile_container const & pc1,
+                         profile_container const & pc2, bool multiple_apps)
 {
-	handle_options(non_options);
+	diff_container dc(pc1, pc2);
+
+	profile_container::symbol_choice choice;
+	choice.threshold = options::threshold;
+
+	diff_collection symbols = dc.get_symbols(choice);
+
+	format_flags flags = get_format_flags(choice.hints);
+	if (multiple_apps)
+		flags = format_flags(flags | ff_app_name);
+
+	format_output::diff_formatter out(dc);
+
+	out.set_nr_classes(nr_classes);
+	out.show_long_filenames(options::long_filenames);
+	out.show_header(options::show_header);
+	out.show_global_percent(options::global_percent);
+	out.vma_format_64bit(choice.hints & cf_64bit_vma);
+	out.add_format(flags);
+
+	options::sort_by.sort(symbols, options::reverse_sort,
+	                      options::long_filenames);
+
+	out.output(cout, symbols);
+}
+
+
+void output_cg_symbols(callgraph_container const & cg, bool multiple_apps)
+{
+	column_flags output_hints = cg.output_hint();
+
+	cg_collection symbols = cg.get_symbols();
+	options::sort_by.sort(symbols, options::reverse_sort,
+	                      options::long_filenames);
+
+	format_output::cg_formatter out(cg);
+
+	out.set_nr_classes(nr_classes);
+	out.show_long_filenames(options::long_filenames);
+	out.show_header(options::show_header);
+	out.vma_format_64bit(output_hints & cf_64bit_vma);
+	out.show_global_percent(options::global_percent);
+
+	format_flags flags = get_format_flags(output_hints);
+	if (multiple_apps)
+		flags = format_flags(flags | ff_app_name);
+
+	out.add_format(flags);
+
+	out.output(cout, symbols);
+}
+
+
+int opreport(options::spec const & spec)
+{
+	handle_options(spec);
 
 	nr_classes = classes.v.size();
 
@@ -405,8 +465,6 @@ int opreport(vector<string> const & non_options)
 		return 0;
 	}
 
-	profile_container samples(options::debug_info, options::details);
-
 	bool multiple_apps = false;
 
 	for (size_t i = 0; i < classes.v.size(); ++i) {
@@ -415,19 +473,65 @@ int opreport(vector<string> const & non_options)
 	}
 
 	list<inverted_profile> iprofiles
-		= invert_profiles(classes, options::extra_found_images);
+		= invert_profiles(options::archive_path, classes,
+				  options::extra_found_images);
 
 	report_image_errors(iprofiles);
 
-	list<inverted_profile>::iterator it = iprofiles.begin();
-	list<inverted_profile>::iterator const end = iprofiles.end();
-
-	for (; it != end; ++it)
-		populate_for_image(samples, *it, options::symbol_filter);
-
 	output_header();
 
-	output_symbols(samples, multiple_apps);
+	if (classes2.v.size()) {
+		for (size_t i = 0; i < classes2.v.size(); ++i) {
+			if (classes2.v[i].profiles.size() > 1)
+				multiple_apps |= true;
+		}
+
+		profile_container pc1(options::debug_info, options::details);
+
+		list<inverted_profile>::iterator it = iprofiles.begin();
+		list<inverted_profile>::iterator const end = iprofiles.end();
+
+		for (; it != end; ++it)
+			populate_for_image(options::archive_path, pc1,
+				*it, options::symbol_filter, 0);
+
+		list<inverted_profile> iprofiles2
+			= invert_profiles(options::archive_path2, classes2,
+				  options::extra_found_images);
+
+		report_image_errors(iprofiles2);
+
+		profile_container pc2(options::debug_info, options::details);
+
+		list<inverted_profile>::iterator it2 = iprofiles2.begin();
+		list<inverted_profile>::iterator const end2 = iprofiles2.end();
+
+		for (; it2 != end2; ++it2)
+			populate_for_image(options::archive_path2, pc2,
+				*it2, options::symbol_filter, 0);
+
+		output_diff_symbols(pc1, pc2, multiple_apps);
+	} else if (options::callgraph) {
+		callgraph_container cg_container;
+		cg_container.populate(options::archive_path, iprofiles,
+			options::extra_found_images,
+			options::debug_info, options::threshold,
+			options::merge_by.lib, options::symbol_filter);
+
+		output_cg_symbols(cg_container, multiple_apps);
+	} else {
+		profile_container samples(options::debug_info, options::details);
+
+		list<inverted_profile>::iterator it = iprofiles.begin();
+		list<inverted_profile>::iterator const end = iprofiles.end();
+
+		for (; it != end; ++it)
+			populate_for_image(options::archive_path, samples,
+				*it, options::symbol_filter, 0);
+
+		output_symbols(samples, multiple_apps);
+	}
+
 	return 0;
 }
 
