@@ -27,10 +27,14 @@
 using namespace std;
 
 profile_classes classes;
+profile_classes classes2;
 
 namespace options {
+	string archive_path;
+	string archive_path2;
 	demangle_type demangle = dmt_normal;
 	bool symbols;
+	bool callgraph;
 	bool debug_info;
 	bool details;
 	bool exclude_dependent;
@@ -56,45 +60,52 @@ vector<string> include_symbols;
 string demangle_option = "normal";
 
 popt::option options_array[] = {
-	popt::option(demangle_option, "demangle", '\0',
-		     "demangle GNU C++ symbol names (default normal)",
-	             "none|normal|smart"),
-	popt::option(outfile, "output-file", 'o',
-	             "output to the given filename", "file"),
-	// PP:5
-	popt::option(options::symbols, "symbols", 'l',
-		     "list all symbols"),
-	popt::option(options::debug_info, "debug-info", 'g',
-		     "add source file and line number to output"),
+	popt::option(options::callgraph, "callgraph", 'c',
+	             "show call graph"),
 	popt::option(options::details, "details", 'd',
 		     "output detailed samples for each symbol"),
-	popt::option(options::exclude_dependent, "exclude-dependent", 'x',
-		     "exclude libs, kernel, and module samples for applications"),
+	popt::option(options::symbols, "symbols", 'l',
+		     "list all symbols"),
+
+	popt::option(outfile, "output-file", 'o',
+	             "output to the given filename", "file"),
+
 	popt::option(sort, "sort", 's',
 		     "sort by", "sample,image,app-name,symbol,debug,vma"),
+	popt::option(options::reverse_sort, "reverse-sort", 'r',
+		     "use reverse sort"),
+	popt::option(mergespec, "merge", 'm',
+		     "comma separated list", "cpu,lib,tid,tgid,unitmask,all"),
+	popt::option(options::exclude_dependent, "exclude-dependent", 'x',
+		     "exclude libs, kernel, and module samples for applications"),
 	popt::option(exclude_symbols, "exclude-symbols", 'e',
 		     "exclude these comma separated symbols", "symbols"),
 	popt::option(include_symbols, "include-symbols", 'i',
 		     "include these comma separated symbols", "symbols"),
-	popt::option(mergespec, "merge", 'm',
-		     "comma separated list", "cpu,lib,tid,tgid,unitmask,all"),
+	popt::option(options::threshold_opt, "threshold", 't',
+		     "minimum percentage needed to produce output",
+		     "percent"),
+
+	popt::option(demangle_option, "demangle", 'D',
+		     "demangle GNU C++ symbol names (default normal)",
+	             "none|normal|smart"),
+	// PP:5
+	popt::option(options::debug_info, "debug-info", 'g',
+		     "add source file and line number to output"),
 	popt::option(options::show_header, "no-header", 'n',
 		     "remove all headers from output"),
 	popt::option(options::show_address, "show-address", 'w',
 	             "show VMA address of each symbol"),
 	popt::option(options::long_filenames, "long-filenames", 'f',
 		     "show the full path of filenames"),
-	popt::option(options::accumulated, "accumulated", 'c',
+	popt::option(options::accumulated, "accumulated", 'a',
 		     "percentage field show accumulated count"),
-	popt::option(options::reverse_sort, "reverse-sort", 'r',
-		     "use reverse sort"),
-	popt::option(options::global_percent, "global-percent", '\0',
+	popt::option(options::global_percent, "global-percent", '%',
 		     "percentage are not relative to symbol count or image "
 		     "count but total sample count"),
 };
 
 
-// FIXME: separate file if reused
 void handle_sort_option()
 {
 	if (sort.empty()) {
@@ -111,7 +122,6 @@ void handle_sort_option()
 }
 
 
-// FIXME: separate file if reused
 void handle_output_file()
 {
 	if (outfile.empty())
@@ -127,17 +137,39 @@ void handle_output_file()
 	cout.rdbuf(os.rdbuf());
 }
 
-/**
- * check incompatible or meaningless options
- *
- */
-void check_options()
+
+///  Check incompatible or meaningless options.
+void check_options(bool diff)
 {
 	using namespace options;
 
 	bool do_exit = false;
 
+	if (callgraph) {
+		symbols = true;
+		if (details) {
+			cerr << "--callgraph is incompatible with --details" << endl;
+			do_exit = true;
+		}
+
+		if (diff) {
+			cerr << "differential profiles are incompatible with --callgraph" << endl;
+			do_exit = true;
+		}
+	}
+
+	if (details && diff) {
+		cerr << "differential profiles are incompatible with --details" << endl;
+		do_exit = true;
+	}
+
 	if (!symbols) {
+		if (diff) {
+			cerr << "different profiles are meaningless "
+				"without --symbols" << endl;
+			do_exit = true;
+		}
+
 		if (show_address) {
 			cerr << "--show-address is meaningless "
 				"without --symbols" << endl;
@@ -164,9 +196,9 @@ void check_options()
 		}
 	}
 
-	if (global_percent && symbols && !details) {
-		cerr << "--global-percent is meaningless "
-		     << "with --symbols and without --details" << endl;
+	if (global_percent && symbols && !(details || callgraph)) {
+		cerr << "--global-percent is meaningless with --symbols "
+		        "and without --details or --callgraph" << endl;
 		do_exit = true;
 	}
 
@@ -174,30 +206,23 @@ void check_options()
 		exit(EXIT_FAILURE);
 }
 
-} // namespace anon
 
-
-void handle_options(vector<string> const & non_options)
+/// process a spec into classes
+string process_spec(profile_classes & classes, list<string> const & spec)
 {
 	using namespace options;
 
-	if (details) {
-		symbols = true;
-		show_address = true;
-	}
+	copy(spec.begin(), spec.end(),
+	     ostream_iterator<string>(cverb << vsfile, " "));
+	cverb << vsfile << "\n\n";
 
-	handle_sort_option();
-	merge_by = handle_merge_option(mergespec, true, exclude_dependent);
-	handle_output_file();
-	demangle = handle_demangle_option(demangle_option);
-	check_options();
+	profile_spec const pspec =
+		profile_spec::create(spec, extra_found_images);
 
-	symbol_filter = string_filter(include_symbols, exclude_symbols);
+	list<string> sample_files = pspec.generate_file_list(exclude_dependent,
+	                                                     !options::callgraph);
 
-	profile_spec const spec =
-		profile_spec::create(non_options, extra_found_images);
-
-	list<string> sample_files = spec.generate_file_list(exclude_dependent, true);
+	cverb << vsfile << "Archive: " << pspec.get_archive_path() << endl;
 
 	cverb << vsfile << "Matched sample files: " << sample_files.size()
 	      << endl;
@@ -213,4 +238,42 @@ void handle_options(vector<string> const & non_options)
 		     "too strict ?" << endl;
 		exit(EXIT_FAILURE);
 	}
+
+	return pspec.get_archive_path();
+}
+
+
+} // namespace anon
+
+
+void handle_options(options::spec const & spec)
+{
+	using namespace options;
+
+	if (details) {
+		symbols = true;
+		show_address = true;
+	}
+
+	handle_sort_option();
+	merge_by = handle_merge_option(mergespec, true, exclude_dependent);
+	handle_output_file();
+	demangle = handle_demangle_option(demangle_option);
+	check_options(spec.first.size());
+
+	symbol_filter = string_filter(include_symbols, exclude_symbols);
+
+	if (!spec.first.size()) {
+		archive_path = process_spec(classes, spec.common);
+	} else {
+		cverb << vsfile << "profile spec 1:" << endl;
+		archive_path = process_spec(classes, spec.first);
+		cverb << vsfile << "profile spec 2:" << endl;
+		archive_path2 = process_spec(classes2, spec.second);
+
+		if (!classes.matches(classes2)) {
+			cerr << "profile classes are incompatible" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
 }
