@@ -17,17 +17,18 @@
 #include <iosfwd>
 
 #include "common_option.h"
-#include "utility.h"
 #include "string_filter.h"
 #include "symbol_sort.h"
 
-class partition_files;
+class profile_classes;
 class merge_option;
 
 namespace options {
-	extern bool demangle;
-	extern bool smart_demangle;
+	extern std::string archive_path;
+	extern std::string archive_path2;
+	extern demangle_type demangle;
 	extern bool symbols;
+	extern bool callgraph;
 	extern bool debug_info;
 	extern bool details;
 	extern bool reverse_sort;
@@ -36,24 +37,22 @@ namespace options {
 	extern merge_option merge_by;
 	extern bool global_percent;
 	extern bool long_filenames;
+	extern bool show_address;
 	extern string_filter symbol_filter;
-	extern double threshold;
 	extern bool show_header;
 	extern bool accumulated;
 }
 
-/**
- * a partition of sample filename to treat, each sub-list is a list of
- * sample to merge. filled by handle_options()
- */
-extern scoped_ptr<partition_files> sample_file_partition;
+/// All the chosen sample files.
+extern profile_classes classes;
+extern profile_classes classes2;
 
 /**
- * get_options - process command line
- * @param non_options vector of non options string
+ * handle_options - process command line
+ * @param spec  profile specification
  *
- * Process the arguments, fatally complaining on error.
+ * Process the spec, fatally complaining on error.
  */
-void handle_options(std::vector<std::string> const & non_options);
+void handle_options(options::spec const & spec);
 
 #endif // OPREPORT_OPTIONS_H
